using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using Magify.Model;
using Cysharp.Threading.Tasks;
using JetBrains.Annotations;

namespace Magify
{
    internal interface IServerApi
    {
        void ChangeServerUrl(EndpointUrl url);

        Task<string> GetAuthorizationTokenUntilSuccessAsync(CancellationToken cancellationToken);
        void ResetAuthorizationToken();

        UniTask<CampaignsContext> GetContextAsync(ConfigScope scopes = default, CancellationToken cancellationToken = default);

        UniTask<bool> SendEvents<T>(EventType eventType, IList<T> events, CancellationToken cancellationToken = default)
            where T : class, IAnalyticsEvent;

        UniTask<PurchaseVerificationResult> VerifyPurchase(PurchaseRecord purchaseRecord, bool waitForResult, CancellationToken cancellationToken);

        /// <returns>
        /// User Id. Can be null
        /// </returns>
        /// <exception cref="OperationCanceledException"></exception>
        UniTask<string> AuthorizeUser([NotNull] string clientId, CancellationToken cancellationToken);
        /// <exception cref="OperationCanceledException"></exception>
        UniTask<bool> SaveState([NotNull] string appState, [CanBeNull] int? weight, CancellationToken cancellationToken);
        /// <exception cref="OperationCanceledException"></exception>
        UniTask<string> RestoreState([CanBeNull] int? weight, CancellationToken cancellationToken);
    }
}