﻿using System.Collections.Generic;
using JetBrains.Annotations;
using Magify.Model;

namespace Magify
{
    internal class PurchasesStorage
    {
        private const string PurchasesFolderName = "purchases";
        private const string PurchasesForVerificationFileName = "purchases_for_verification.json";

        [NotNull]
        private readonly FileStorage _storage;
        public string RootFolderPath { get; }

        public PurchasesStorage(string storagePath)
        {
            _storage = new FileStorage(PurchasesFolderName, storagePath);
            RootFolderPath = storagePath;
        }

        [NotNull]
        public List<PurchaseRecord> LoadRecords()
        {
            return _storage.LoadJson<List<PurchaseRecord>>(PurchasesForVerificationFileName)
                ?? new List<PurchaseRecord>();
        }

        public void SaveRecords([NotNull] IReadOnlyCollection<PurchaseRecord> unsentPurchaseRecords)
        {
            var data = JsonFacade.SerializeObject(unsentPurchaseRecords);
            _storage.Save(PurchasesForVerificationFileName, data);
        }

        public bool Exists()
        {
            return _storage.Exists(PurchasesForVerificationFileName);
        }
    }
}