﻿using System.Threading;
using JetBrains.Annotations;
using NUnit.Framework;

namespace Magify.Tests
{
    internal partial class MagifyClientTests
    {
        private const string AppName = "TestApp";
        private const string ConfigPath = "";
        private const Environment OfflineEnvironment = Environment.Offline;

        internal class MagifyClientTestsPart
        {
            [SetUp]
            [TearDown]
            public void ClearAllData()
            {
                EditorModeTestsEnvironment.Clear();
            }

            protected CancellationToken GetCancellationToken(int cancelDelay = 1000)
            {
                return new CancellationTokenSource(cancelDelay).Token;
            }
        }

        [NotNull]
        private static MagifyClient CreateDefault(INetworkClient networkClient = default, EditorDevice editorDevice = default, Environment? environment = null)
        {
            if (networkClient == null)
            {
                return new MagifyClient(AppName, ConfigPath, EditorModeTestsEnvironment.RootStoragePath, new ClientStateConfig(), environment ?? OfflineEnvironment, editorDevice);
            }
            return new MagifyClient(AppName, ConfigPath, EditorModeTestsEnvironment.RootStoragePath, new ClientStateConfig(), environment ?? OfflineEnvironment, editorDevice, networkClient);
        }

        [NotNull]
        private static MagifyClient CreateDefaultAndInit()
        {
            var client = new MagifyClient(AppName, ConfigPath, EditorModeTestsEnvironment.RootStoragePath, EditorModeTestsEnvironment.DefaultClientStateConfig, OfflineEnvironment);
            client.InitializeSdk();
            client.Setup();
            return client;
        }

        private static MagifyClient CreateWithDeviceAndInit(EditorDevice editorDevice)
        {
            var client = new MagifyClient(AppName, ConfigPath, EditorModeTestsEnvironment.RootStoragePath, EditorModeTestsEnvironment.DefaultClientStateConfig, OfflineEnvironment, editorDevice);
            client.InitializeSdk();
            client.Setup();
            return client;
        }

        [NotNull]
        private static CampaignsTracker GetCampaignsTracker([NotNull] MagifyClient magifyClient)
        {
            return GetPrivateField<CampaignsTracker>(magifyClient, "_campaignsTracker");
        }

        [NotNull]
        private static AnalyticsTracker GetAnalyticsTracker([NotNull] MagifyClient magifyClient)
        {
            return GetPrivateField<AnalyticsTracker>(magifyClient, "_analyticsTracker");
        }

        [NotNull]
        private static PlatformAPI GetPlatformAPI([NotNull] MagifyClient magifyClient)
        {
            return GetPrivateField<PlatformAPI>(magifyClient, "_platform");
        }

        [NotNull]
        private static GeneralPrefs GetGeneralPrefs([NotNull] MagifyClient magifyClient)
        {
            return GetPrivateField<GeneralPrefs>(magifyClient, "_generalPrefs");
        }

        [NotNull]
        private static SessionCounter GetSessionCounter([NotNull] MagifyClient magifyClient)
        {
            return GetPrivateField<SessionCounter>(magifyClient, "_sessionCounter");
        }

		[NotNull]
        private static ServerApi GetServerApi([NotNull] MagifyClient magifyClient)
        {
            return GetPrivateField<ServerApi>(magifyClient, "_serverApi");
        }

        [NotNull]
        private static ContextApplicator GetContextApplicator([NotNull] MagifyClient magifyClient)
        {
            return GetPrivateField<ContextApplicator>(magifyClient, "_contextApplicator");
        }

		[NotNull]
        private static T GetPrivateField<T>([NotNull] MagifyClient client, [NotNull] string fieldName)
            where T : class
        {
            return (T)client
                    .GetType()
                    .GetField(fieldName, System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance)
                    !.GetValue(client)!;
        }
    }
}